package com.example;

import genericAccessorMethods.GenericAccessorMethodsPerformanceTestDispatcher;
import genericAttributes.GenericAttributesPerformanceTestDispatcher;
import genericUnitypeAttributes.GenericUnitypeAttributesPerformanceTestDispatcher;

import java.util.Arrays;
import java.util.List;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;

import com.example.IPerformanceTestDispatcher.AccessType;

@RunWith(Parameterized.class)
public class PerformanceTest {
	private static final int COUNT = 100000;
	private static final int ITERATIONS = 10000;

	private final IPerformanceTestDispatcher dispatcher;

	@Parameters
	public static List<Object[]> data() {
		return Arrays.asList(new Object[][] {
				new Object[] {
						"GenericAccessorMethods // generic",
						new GenericAccessorMethodsPerformanceTestDispatcher(
								COUNT), AccessType.GENERIC },
				new Object[] {
						"GenericAccessorMethods // specific",
						new GenericAccessorMethodsPerformanceTestDispatcher(
								COUNT), AccessType.SPECIFIC },
				new Object[] { "GenericAttributes // generic",
						new GenericAttributesPerformanceTestDispatcher(COUNT),
						AccessType.GENERIC },
				new Object[] { "GenericAttributes // specific",
						new GenericAttributesPerformanceTestDispatcher(COUNT),
						AccessType.SPECIFIC },
				new Object[] {
						"GenericUnitypeAttributes // generic",
						new GenericUnitypeAttributesPerformanceTestDispatcher(
								COUNT), AccessType.GENERIC },
				new Object[] {
						"GenericUnitypeAttributes // specific",
						new GenericUnitypeAttributesPerformanceTestDispatcher(
								COUNT), AccessType.SPECIFIC } });
	}

	public PerformanceTest(String label, IPerformanceTestDispatcher dispatcher,
			AccessType accessType) {
		this.dispatcher = dispatcher;
		dispatcher.setAccessType(accessType);
	}

	@Test
	public void testStatistics() {
		for (int i = 0; i < ITERATIONS; i++) {
			dispatcher.computeStatistics();
		}
	}

	@Test
	public void testInstanceof() {
		for (int i = 0; i < ITERATIONS; i++) {
			dispatcher.computeInstanceof();
		}
	}

	@Test
	public void testDispatch() {
		for (int i = 0; i < ITERATIONS; i++) {
			dispatcher.computeDispatch();
		}
	}
}
