package com.example;

import genericAccessorMethods.GenericAccessorMethodsCountable;
import genericAttributes.GenericAttributesCountable;

public class PerformanceTestOld {
	private static final int count = 10000;
	private static final int iterations = 1000;

	public static genericAttributes.BookLibrary createGenericAttributeBookLibrary(
			int count) {
		return genericAttributes.GenericAttributesTest.createBookLibrary(count);
	}

	public static genericAccessorMethods.BookLibrary createGenericAccessorMethodsBookLibrary(
			int count) {
		return genericAccessorMethods.GenericAccessorMethodsTest
				.createBookLibrary(count);
	}

	public static void main(String[] args) {
		new PerformanceTestOld().runTests();
	}

	private void runTests() {
		System.out.println("Count: " + count);
		System.out.println("Iteration: " + iterations);

		comparePerformance("Statistics",
				new genericAccessorMethods.LibraryStatistics(),
				new genericAttributes.LibraryStatistics());
		comparePerformance("StatisticsInstanceof",
				new genericAccessorMethods.LibraryStatisticsInstanceof(),
				new genericAttributes.LibraryStatisticsInstanceof());
		comparePerformance("StatisticsDispatch",
				new genericAccessorMethods.LibraryStatisticsDispatch(),
				new genericAttributes.LibraryStatisticsDispatch());
	}

	private void comparePerformance(String msg,
			GenericAccessorMethodsCountable genericAccessorMethodsCountable,
			GenericAttributesCountable genericAttributesCountable) {
		long genericAccessorMethodsTimingGeneric = 0;
		long genericAttributesTimingGeneric = 0;

		long genericAccessorMethodsTimingSpecific = 0;
		long genericAttributesTimingSpecific = 0;

		for (int i = 0; i < iterations; i++) {
			genericAccessorMethodsTimingGeneric += timeGenericAccessorMethodsCountableGeneric(genericAccessorMethodsCountable);
			genericAttributesTimingGeneric += timeGenericAttributesCountableGeneric(genericAttributesCountable);

			genericAccessorMethodsTimingSpecific += timeGenericAccessorMethodsCountableSpecific(genericAccessorMethodsCountable);
			genericAttributesTimingSpecific += timeGenericAttributesCountableSpecific(genericAttributesCountable);
		}

		genericAccessorMethodsTimingGeneric /= iterations;
		genericAttributesTimingGeneric /= iterations;
		double factorGeneric = ((double) genericAttributesTimingGeneric)
				/ genericAccessorMethodsTimingGeneric;

		genericAccessorMethodsTimingSpecific /= iterations;
		genericAttributesTimingSpecific /= iterations;
		double factorSpecific = ((double) genericAttributesTimingSpecific)
				/ genericAccessorMethodsTimingSpecific;

		System.out
				.println(String
						.format("Comparing %20s: GenericAccessorMethods %7d vs. %7d GenericAttributes (factor %7.2f); *Specific: %7d vs. %7d (factor %7.2f)",
								msg, genericAccessorMethodsTimingGeneric,
								genericAttributesTimingGeneric, factorGeneric,
								genericAccessorMethodsTimingSpecific,
								genericAttributesTimingSpecific, factorSpecific));
	}

	private long timeGenericAccessorMethodsCountableGeneric(
			GenericAccessorMethodsCountable genericAccessorMethodsCountable) {
		genericAccessorMethods.BookLibrary library = createGenericAccessorMethodsBookLibrary(count);

		final long startTime = System.nanoTime();
		final long endTime;
		try {
			genericAccessorMethodsCountable.countEntries(library);
		} finally {
			endTime = System.nanoTime();
		}
		final long duration = endTime - startTime;

		return duration;
	}

	private long timeGenericAttributesCountableGeneric(
			GenericAttributesCountable genericAttributesCountable) {
		genericAttributes.BookLibrary library = createGenericAttributeBookLibrary(count);

		final long startTime = System.nanoTime();
		final long endTime;
		try {
			genericAttributesCountable.countEntries(library);
		} finally {
			endTime = System.nanoTime();
		}
		final long duration = endTime - startTime;

		return duration;
	}

	private long timeGenericAccessorMethodsCountableSpecific(
			GenericAccessorMethodsCountable genericAccessorMethodsCountable) {
		genericAccessorMethods.BookLibrary library = createGenericAccessorMethodsBookLibrary(count);

		final long startTime = System.nanoTime();
		final long endTime;
		try {
			genericAccessorMethodsCountable.countBooks(library);
		} finally {
			endTime = System.nanoTime();
		}
		final long duration = endTime - startTime;

		return duration;
	}

	private long timeGenericAttributesCountableSpecific(
			GenericAttributesCountable genericAttributesCountable) {
		genericAttributes.BookLibrary library = createGenericAttributeBookLibrary(count);

		final long startTime = System.nanoTime();
		final long endTime;
		try {
			genericAttributesCountable.countBooks(library);
		} finally {
			endTime = System.nanoTime();
		}
		final long duration = endTime - startTime;

		return duration;
	}
}
