package genericAccessorMethods;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.emf.common.util.EList;
import org.junit.Test;

public class GenericAccessorMethodsTest {
	public static MovieLibrary createMovieLibrary(int movieCount) {
		MovieLibrary result = GenericAccessorMethodsFactory.eINSTANCE
				.createMovieLibrary();

		for (int i = 0; i < movieCount; i++) {
			result.getMovies().add(createMovie());
		}

		return result;
	}

	public static Movie createMovie() {
		return GenericAccessorMethodsFactory.eINSTANCE.createMovie();
	}

	public static Collection<Movie> createMovies(int count) {
		ArrayList<Movie> result = new ArrayList<Movie>(count);
		for (int i = 0; i < count; i++) {
			result.add(createMovie());
		}

		return result;
	}

	public static SciFi createSciFi() {
		return GenericAccessorMethodsFactory.eINSTANCE.createSciFi();
	}

	public static Collection<SciFi> createSciFis(int count) {
		ArrayList<SciFi> result = new ArrayList<SciFi>(count);
		for (int i = 0; i < count; i++) {
			result.add(createSciFi());
		}

		return result;
	}

	public static Comedy createComedy() {
		return GenericAccessorMethodsFactory.eINSTANCE.createComedy();
	}

	public static Collection<Comedy> createComedys(int count) {
		ArrayList<Comedy> result = new ArrayList<Comedy>(count);
		for (int i = 0; i < count; i++) {
			result.add(createComedy());
		}

		return result;
	}

	public static Collection<Movie> createMixedMovies(int count) {
		ArrayList<Movie> result = new ArrayList<Movie>(count);

		result.addAll(createMovies(count / 3));
		result.addAll(createSciFis(count / 3));
		result.addAll(createComedys(count / 3));

		while (result.size() < count) {
			result.add(createMovie());
		}
		return result;
	}

	public static BookLibrary createBookLibrary(int bookCount) {
		BookLibrary result = GenericAccessorMethodsFactory.eINSTANCE
				.createBookLibrary();

		for (int i = 0; i < bookCount; i++) {
			result.getBooks().add(createBook());
		}

		return result;
	}

	public static Book createBook() {
		return GenericAccessorMethodsFactory.eINSTANCE.createBook();
	}

	public static Collection<Book> createBooks(int count) {
		ArrayList<Book> result = new ArrayList<Book>(count);
		for (int i = 0; i < count; i++) {
			result.add(createBook());
		}

		return result;
	}

	@Test
	public void countMoviesLibraryStatistics() {
		assertEquals(2,
				new LibraryStatistics().countEntries(createMovieLibrary(2)));
	}

	@Test
	public void countBooksLibraryStatistics() {
		assertEquals(23,
				new LibraryStatistics().countEntries(createBookLibrary(23)));
	}

	@Test
	public void countMoviesLibraryStatisticsInstanceof() {
		assertEquals(42,
				new LibraryStatisticsInstanceof()
						.countEntries(createMovieLibrary(42)));
	}

	@Test
	public void countBooksLibraryStatisticsInstanceof() {
		assertEquals(1337,
				new LibraryStatisticsInstanceof()
						.countEntries(createBookLibrary(1337)));
	}

	@Test
	public void countMoviesLibraryStatisticsDispatch() {
		assertEquals(31337,
				new LibraryStatisticsDispatch()
						.countEntries(createMovieLibrary(31337)));
	}

	@Test
	public void countBooksLibraryStatisticsDispatch() {
		assertEquals(4711,
				new LibraryStatisticsDispatch()
						.countEntries(createBookLibrary(4711)));
	}

	@Test(expected=java.lang.Error.class)
	public void addBookToBookLibraryGeneric() {
		BookLibrary library = createBookLibrary(0);
		EList<? extends AbstractEntry> entries = library.getEntries();
		// Illegal -- why?
		assertTrue(entries.add(createBook()));
		assertEquals(1, library.getEntries().size());
		assertEquals(1, library.getBooks().size());
	}

	@Test(expected=java.lang.Error.class)
	public void addBookToMovieLibraryGeneric() {
		MovieLibrary library = createMovieLibrary(0);
		EList<? extends AbstractEntry> entries = library.getEntries();
		// Illegal -- why?
		assertFalse(entries.add(createBook()));
		assertEquals(0, library.getEntries().size());
		assertEquals(0, library.getMovies().size());
	}

	@Test
	public void addBookToBookLibrarySpecific() {
		BookLibrary library = createBookLibrary(0);
		EList<Book> books = library.getBooks();
		assertTrue(books.add(createBook()));
		assertEquals(1, library.getEntries().size());
		assertEquals(1, library.getBooks().size());
	}

	@Test
	public void addBookToMovieLibrarySpecific() {
		MovieLibrary library = createMovieLibrary(0);
		EList<Movie> movies = library.getMovies();
		// Illegal
//		 assertFalse(movies.add(createBook()));
		assertEquals(0, library.getEntries().size());
		assertEquals(0, library.getMovies().size());
	}

	@Test
	public void addMovieToBookLibrarySpecific() {
		BookLibrary library = createBookLibrary(0);
		EList<Book> books = library.getBooks();
		// Illegal
//		 assertFalse(books.add(createMovie()));
		assertEquals(0, library.getEntries().size());
		assertEquals(0, library.getBooks().size());
	}

	@Test(expected=java.lang.Error.class)
	public void addBooksToBookLibraryGeneric() {
		BookLibrary library = createBookLibrary(0);
		EList<? extends AbstractEntry> entries = library.getEntries();
		// Illegal -- why?
		assertTrue(entries.addAll(createBooks(23)));
		assertEquals(23, library.getEntries().size());
		assertEquals(23, library.getBooks().size());
	}

	@Test
	public void addBooksToMovieLibraryGeneric() {
		MovieLibrary library = createMovieLibrary(0);
		EList<? extends AbstractEntry> entries = library.getEntries();
		// Illegal
//		 assertFalse(entries.addAll(createBooks(423)));
		assertEquals(0, library.getEntries().size());
		assertEquals(0, library.getMovies().size());
	}

	@Test
	public void addBooksToBookLibrarySpecific() {
		BookLibrary library = createBookLibrary(0);
		EList<Book> books = library.getBooks();
		assertTrue(books.addAll(createBooks(42)));
		assertEquals(42, library.getEntries().size());
		assertEquals(42, library.getBooks().size());
	}

	@Test
	public void addBooksToMovieLibrarySpecific() {
		MovieLibrary library = createMovieLibrary(0);
		EList<Movie> movies = library.getMovies();
		// Illegal
		// assertFalse(movies.addAll(createBooks(1337)));
		assertEquals(0, library.getEntries().size());
		assertEquals(0, library.getMovies().size());
	}

	@Test
	public void addMoviesToBookLibrarySpecific() {
		BookLibrary library = createBookLibrary(0);
		EList<Book> books = library.getBooks();
		// Illegal
		// assertFalse(books.addAll(createMovies(23)));
		assertEquals(0, library.getEntries().size());
		assertEquals(0, library.getBooks().size());
	}

	@Test(expected=java.lang.Error.class)
	public void addSciFiToBookLibraryGeneric() {
		BookLibrary library = createBookLibrary(0);
		EList<? extends AbstractEntry> entries = library.getEntries();
		// Illegal -- why?
		assertFalse(entries.add(createSciFi()));
		assertEquals(0, library.getEntries().size());
		assertEquals(0, library.getBooks().size());
	}

	@Test(expected=java.lang.Error.class)
	public void addSciFiToMovieLibraryGeneric() {
		MovieLibrary library = createMovieLibrary(0);
		EList<? extends AbstractEntry> entries = library.getEntries();
		// Illegal -- why?
		assertTrue(entries.add(createSciFi()));
		assertEquals(1, library.getEntries().size());
		assertEquals(1, library.getMovies().size());
	}

	@Test(expected=java.lang.Error.class)
	public void addMixedMovieToMovieLibraryGeneric() {
		MovieLibrary library = createMovieLibrary(0);
		EList<? extends AbstractEntry> entries = library.getEntries();
		// Illegal -- why?
		assertTrue(entries.add(createMovie()));
		assertTrue(entries.add(createSciFi()));
		assertTrue(entries.add(createComedy()));
		assertEquals(3, library.getEntries().size());
		assertEquals(3, library.getMovies().size());
	}

	@Test
	public void addSciFiToMovieLibrarySpecific() {
		MovieLibrary library = createMovieLibrary(0);
		EList<Movie> movies = library.getMovies();
		assertTrue(movies.add(createSciFi()));
		assertEquals(1, library.getEntries().size());
		assertEquals(1, library.getMovies().size());
	}

	@Test
	public void addSciFiToBookLibrarySpecific() {
		BookLibrary library = createBookLibrary(0);
		EList<Book> books = library.getBooks();
		// Illegal
		// assertFalse(books.add(createSciFi()));
		assertEquals(0, library.getEntries().size());
		assertEquals(0, library.getBooks().size());
	}

	@Test
	public void addMixedMovieToMovieLibrarySpecific() {
		MovieLibrary library = createMovieLibrary(0);
		EList<Movie> movies = library.getMovies();
		assertTrue(movies.add(createMovie()));
		assertTrue(movies.add(createSciFi()));
		assertTrue(movies.add(createComedy()));
		assertEquals(3, library.getEntries().size());
		assertEquals(3, library.getMovies().size());
	}

	@Test(expected=java.lang.Error.class)
	public void addSciFisToBookLibraryGeneric() {
		BookLibrary library = createBookLibrary(0);
		EList<? extends AbstractEntry> entries = library.getEntries();
		// Illegal -- why?
		assertFalse(entries.addAll(createSciFis(2001)));
		assertEquals(0, library.getEntries().size());
		assertEquals(0, library.getBooks().size());
	}

	@Test(expected=java.lang.Error.class)
	public void addSciFisToMovieLibraryGeneric() {
		MovieLibrary library = createMovieLibrary(0);
		EList<? extends AbstractEntry> entries = library.getEntries();
		// Illegal -- why?
		assertTrue(entries.addAll(createSciFis(2021)));
		assertEquals(2021, library.getEntries().size());
		assertEquals(2021, library.getMovies().size());
	}

	@Test(expected=java.lang.Error.class)
	public void addMixedMoviesToMovieLibraryGeneric() {
		MovieLibrary library = createMovieLibrary(0);
		EList<? extends AbstractEntry> entries = library.getEntries();
		// Illegal -- why?
		assertTrue(entries.addAll(createMixedMovies(10000)));
		assertEquals(10000, library.getEntries().size());
		assertEquals(10000, library.getMovies().size());
	}

	@Test
	public void addSciFisToMovieLibrarySpecific() {
		MovieLibrary library = createMovieLibrary(0);
		EList<Movie> movies = library.getMovies();
		assertTrue(movies.addAll(createSciFis(451)));
		assertEquals(451, library.getEntries().size());
		assertEquals(451, library.getMovies().size());
	}

	@Test
	public void addSciFisToBookLibrarySpecific() {
		BookLibrary library = createBookLibrary(0);
		EList<Book> books = library.getBooks();
		// Illegal
		// assertFalse(books.addAll(createSciFis(911)));
		assertEquals(0, library.getEntries().size());
		assertEquals(0, library.getBooks().size());
	}

	@Test
	public void addMixedMoviesToMovieLibrarySpecific() {
		MovieLibrary library = createMovieLibrary(0);
		EList<Movie> movies = library.getMovies();
		assertTrue(movies.addAll(createMixedMovies(2525)));
		assertEquals(2525, library.getEntries().size());
		assertEquals(2525, library.getMovies().size());
	}

}
